/**
 * @hidden
 */
/// <reference types="node" />
/**
 * File contains vars used to establish service connection between client and provider.
 *
 * These are separated out from 'internal.ts' as including these from provider code will cause the provider to connect
 * to itself.
 *
 * These types are a part of the client, but are not required by applications wishing to interact with the service.
 * This file is excluded from the public-facing TypeScript documentation.
 */
import { EventEmitter } from 'events';
import { ChannelClient } from 'hadouken-js-adapter/out/types/src/api/interappbus/channel/client';
import { SnapAndDockEvent } from '../client/snapanddock';
import { TabbingEvent } from '../client/tabbing';
import { TabstripEvent } from '../client/tabstrip';
import { WorkspacesEvent } from '../client/workspaces';
import { APITopic } from './internal';
/**
 * Defines all events that are fired by the service
 */
export declare type LayoutsEvent = TabstripEvent | TabbingEvent | WorkspacesEvent | SnapAndDockEvent;
/**
 * The event emitter to emit events received from the service.  All addEventListeners will tap into this.
 */
export declare const eventEmitter: EventEmitter;
export declare function getServicePromise(): Promise<ChannelClient>;
/**
 * Wrapper around service.dispatch to help with type checking
 */
export declare function tryServiceDispatch<T, R>(action: APITopic, payload?: T): Promise<R>;
